<?php defined('SYSPATH') or die('No direct script access.');
 
class Controller_Handler extends Controller {
    protected $user, $post, $get, $model;
    private $access;
    
    const ACCESS_USER = 0; 
    const ACCESS_GUEST = 1; 
    const ACCESS_ANYBODY = 2; 

    public function before()
    {
        $isDownload = Cookie::get('fD');
        $user = User::instance();
        // Если у нас запрос идет не из AJAX
        if(!$this->request->is_ajax() && !$isDownload)
        {
            // Перенаправляем на ошибку доступа
            throw HTTP_Exception::factory (403);   
        }
        
        // Обработка POST-данных
        $this->post = Validation::factory(Arr::map('trim', $_POST));
        $this->get = Validation::factory(Arr::map('trim', $_GET));
        
        // Если авторизован, получаем данные аккаунта
        if($user->isSignedIn()) 
        {
            $this->user = $user->getInfoAsArray();
        }
        
        // Получаем имя маршрута 
        $route = Route::name($this->request->route());
        $route .= ':'.$this->request->controller();
        $userMark = $user->offsetGet('RoleMark');
        // Если запрос не прошел на проверку доступа
        if( !$this->checkAccessLevel() || 
            !$this->checkBitmask($userMark, $route))
        {
            // Перенаправляем на ошибку доступа
            throw HTTP_Exception::factory (403);
        }
    }
    
    protected function setAccessLevel($level)
    {
        $this->access = (int) $level;
    }
    
    protected function checkBitmask($userMark, $route)
    {
        $sysModel = new Model_System;
        $bitmask = $sysModel->getBitmaskForRoute($route);
        if(!$bitmask)
            return true;
        return ($bitmask & $userMark) != 0;
    }    
    
    protected function checkAccessLevel()
    {
        switch($this->access)
        {
            case self::ACCESS_USER:
                return User::instance()->isSignedIn();            
            break;
        
            case self::ACCESS_GUEST:
                return !User::instance()->isSignedIn();            
            break;
        
            case self::ACCESS_ANYBODY:
                return TRUE;
            break;
        }
    }
}
